package com.yizhi.library.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.library.application.domain.Favorite;
import com.yizhi.library.application.mapper.FavoriteMapper;
import com.yizhi.library.application.service.IFavoriteService;
import com.yizhi.library.application.vo.FavoriteVO;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 收藏记录表 服务实现类
 * </p>
 *
 * @author dingxiaowei123
 * @since 2019-07-12
 */
@Service
public class FavoriteServiceImpl extends ServiceImpl<FavoriteMapper, Favorite> implements IFavoriteService {

    @Autowired
    private IdGenerator idGenerator;


    @Override
    public boolean favorite(Long studentCaseId) {

        RequestContext res = ContextHolder.get();

        Favorite favorite = new Favorite();
        favorite.setId(idGenerator.generate());
        favorite.setBizId(studentCaseId);
        favorite.setAccountId(res.getAccountId());
        favorite.setType(1);
        favorite.setState(1);
        favorite.setCreateById(res.getAccountId());
        favorite.setCreateByName(res.getAccountName());
        favorite.setCreateTime(new Date());
        favorite.setUpdateById(res.getAccountId());
        favorite.setUpdateByName(res.getAccountName());
        favorite.setUpdateTime(new Date());
        favorite.setCompanyId(res.getCompanyId());
        favorite.setSiteId(res.getSiteId());
        favorite.setOrgId(res.getOrgId());

        return this.insert(favorite);
    }

    @Override
    public boolean cancelFavorite(Long studentCaseId) {

        RequestContext res = ContextHolder.get();

        Favorite favorite = new Favorite();
        favorite.setBizId(studentCaseId);
        favorite.setAccountId(res.getAccountId());
        favorite.setType(1);

        return this.delete(new EntityWrapper<>(favorite));
    }


    @Override
    public boolean favoriteStudentCase(Long studentCaseId){
        RequestContext res = ContextHolder.get();
        Favorite favorite = new Favorite();
        favorite.setBizId(studentCaseId);
        favorite.setAccountId(res.getAccountId());
        favorite.setState(1);
       int count = this.selectCount(new EntityWrapper<>(favorite));
       return count>0;
    }

    @Override
    public Page<FavoriteVO> getFavoriteList(Integer pageNo, Integer pageSize) {
        RequestContext res = ContextHolder.get();
        Favorite favorite = new Favorite();
        favorite.setAccountId(res.getAccountId());
        favorite.setState(1);
        EntityWrapper<Favorite> entityWrapper = new EntityWrapper<Favorite>(favorite);
        List<String> list = Lists.newArrayList();
        list.add("createTime");
        entityWrapper.orderDesc(list);
        entityWrapper.setSqlSelect("DISTINCT biz_id,MAX(create_time) AS createTime");
        entityWrapper.groupBy("biz_id");
        Page<Favorite>  page = this.selectPage(new Page<>(pageNo,pageSize),entityWrapper);
        Page<FavoriteVO> voPage =  new Page<>(pageNo,pageSize);
        if(page!=null) {
            voPage.setTotal(page.getTotal());
            if(CollectionUtils.isNotEmpty(page.getRecords())){
                List<FavoriteVO> favoriteVOList = Lists.newArrayList();
                page.getRecords().stream().forEach(f->{
                    FavoriteVO vo = new FavoriteVO();
                    vo.setFavoriteId(f.getBizId());
                    vo.setSubmitTime(f.getCreateTime());
                    BeanUtils.copyProperties(f,vo);
                    favoriteVOList.add(vo);
                });
                voPage.setRecords(favoriteVOList);
            }
        }
        return voPage;
    }
}
